@@ -23,8 +23,9 @@
 ## See license_change file for details.
 
 import gettext
-from gi.repository import GObject, Peas, Totem # pylint: disable-msg=E0611
-import dbus, dbus.service
+from gi.repository import GObject, Peas, Totem # pylint: disable=no-name-in-module
+import dbus
+import dbus.service
 from dbus.mainloop.glib import DBusGMainLoop
 
 gettext.textdomain ("totem")
@@ -51,7 +52,7 @@ class DbusService (GObject.Object, Peas.Activatable):
         # Ensure we don't leak our paths on the bus
         self.root.disconnect ()
 
-class Root (dbus.service.Object): # pylint: disable-msg=R0923,R0904
+class Root (dbus.service.Object): # pylint: disable=R0904
     def __init__ (self, name, totem):
         dbus.service.Object.__init__ (self, name, '/org/mpris/MediaPlayer2')
         self.totem = totem
@@ -84,15 +85,14 @@ class Root (dbus.service.Object): # pylint: disable-msg=R0923,R0904
     def __calculate_playback_status (self):
         if self.totem.is_playing ():
             return 'Playing'
-        elif self.totem.is_paused ():
+        if self.totem.is_paused ():
             return 'Paused'
-        else:
-            return 'Stopped'
+        return 'Stopped'
 
     def __calculate_metadata (self):
         metadata = {
             'mpris:trackid': dbus.String (self.totem.props.current_mrl,
-                variant_level = 1),
+                                          variant_level = 1),
             'mpris:length': dbus.Int64 (
                 self.totem.props.stream_length * 1000,
                 variant_level = 1),
@@ -116,7 +116,7 @@ class Root (dbus.service.Object): # pylint: disable-msg=R0923,R0904
 
         return metadata
 
-    def __do_update_metadata (self, _, artist, # pylint: disable-msg=R0913
+    def __do_update_metadata (self, _, artist, # pylint: disable=R0913
                               title, album, num):
         self.current_metadata = self.null_metadata.copy ()
         if title:
@@ -129,23 +129,25 @@ class Root (dbus.service.Object): # pylint: disable-msg=R0923,R0904
             self.current_metadata['tracknumber'] = num
 
         self.PropertiesChanged ('org.mpris.MediaPlayer2.Player',
-            { 'Metadata': self.__calculate_metadata () }, [])
+                                { 'Metadata': self.__calculate_metadata () }, [])
 
-    def __do_notify_playing (self, _, prop): # pylint: disable-msg=W0613
+    def __do_notify_playing (self, _, prop): # pylint: disable=W0613
         self.PropertiesChanged ('org.mpris.MediaPlayer2.Player',
-            { 'PlaybackStatus': self.__calculate_playback_status () }, [])
+                                { 'PlaybackStatus': self.__calculate_playback_status () }, [])
 
-    def __do_notify_current_mrl (self, _, prop): # pylint: disable-msg=W0613
+    def __do_notify_current_mrl (self, _, prop): # pylint: disable=W0613
         self.PropertiesChanged ('org.mpris.MediaPlayer2.Player', {
-            'CanPlay': (self.totem.props.current_mrl != None),
-            'CanPause': (self.totem.props.current_mrl != None),
-            'CanSeek': (self.totem.props.current_mrl != None and
+            'CanPlay': (self.totem.props.current_mrl is not None),
+            'CanPause': (self.totem.props.current_mrl is not None),
+            'CanSeek': (self.totem.props.current_mrl is not None and
                         self.totem.props.seekable),
+            'CanGoNext': self.totem.can_seek_next (),
+            'CanGoPrevious': self.totem.can_seek_previous (),
         }, [])
 
-    def __do_notify_seekable (self, _, prop): # pylint: disable-msg=W0613
+    def __do_notify_seekable (self, _, prop): # pylint: disable=W0613
         self.PropertiesChanged ('org.mpris.MediaPlayer2.Player', {
-            'CanSeek': (self.totem.props.current_mrl != None and
+            'CanSeek': (self.totem.props.current_mrl is not None and
                         self.totem.props.seekable),
         }, [])
 
@@ -158,15 +160,15 @@ class Root (dbus.service.Object): # pylint: disable-msg=R0923,R0904
 
     # org.freedesktop.DBus.Properties interface
     @dbus.service.method (dbus_interface = dbus.PROPERTIES_IFACE,
-                          in_signature = 'ss', # pylint: disable-msg=C0103
+                          in_signature = 'ss', # pylint: disable=C0103
                           out_signature = 'v')
-    def Get (self, interface_name, property_name): # pylint: disable-msg=C0103
+    def Get (self, interface_name, property_name): # pylint: disable=C0103
         return self.GetAll (interface_name)[property_name]
 
     @dbus.service.method (dbus_interface = dbus.PROPERTIES_IFACE,
-                          in_signature = 's', # pylint: disable-msg=C0103
+                          in_signature = 's', # pylint: disable=C0103
                           out_signature = 'a{sv}')
-    def GetAll (self, interface_name): # pylint: disable-msg=C0103
+    def GetAll (self, interface_name): # pylint: disable=C0103
         if interface_name == 'org.mpris.MediaPlayer2':
             return {
                 'CanQuit': True,
@@ -177,10 +179,10 @@ class Root (dbus.service.Object): # pylint: disable-msg=R0923,R0904
                 'SupportedUriSchemes': self.totem.get_supported_uri_schemes (),
                 'SupportedMimeTypes': self.totem.get_supported_content_types (),
             }
-        elif interface_name == 'org.mpris.MediaPlayer2.Player':
+
+        if interface_name == 'org.mpris.MediaPlayer2.Player':
             # Loop status (we don't support Track)
-            if self.totem.action_remote_get_setting (
-                Totem.RemoteSetting.REPEAT):
+            if self.totem.remote_get_setting (Totem.RemoteSetting.REPEAT):
                 loop_status = 'Playlist'
             else:
                 loop_status = 'None'
@@ -194,11 +196,11 @@ class Root (dbus.service.Object): # pylint: disable-msg=R0923,R0904
                 'Metadata': self.__calculate_metadata (),
                 'Volume': self.totem.get_volume (), # TODO: Notifications
                 'Position': dbus.Int64(self.totem.props.current_time * 1000),
-                'CanGoNext': True, # TODO
-                'CanGoPrevious': True, # TODO
-                'CanPlay': (self.totem.props.current_mrl != None),
-                'CanPause': (self.totem.props.current_mrl != None),
-                'CanSeek': (self.totem.props.current_mrl != None and
+                'CanGoNext': self.totem.can_seek_next (),
+                'CanGoPrevious': self.totem.can_seek_previous (),
+                'CanPlay': (self.totem.props.current_mrl is not None),
+                'CanPause': (self.totem.props.current_mrl is not None),
+                'CanSeek': (self.totem.props.current_mrl is not None and
                             self.totem.props.seekable),
                 'CanControl': True,
             }
@@ -206,11 +208,11 @@ class Root (dbus.service.Object): # pylint: disable-msg=R0923,R0904
         raise dbus.exceptions.DBusException (
             'org.mpris.MediaPlayer2.UnknownInterface',
             _('The MediaPlayer2 object does not implement the ‘%s’ interface')
-                % interface_name)
+            % interface_name)
 
     @dbus.service.method (dbus_interface = dbus.PROPERTIES_IFACE,
-                          in_signature = 'ssv') # pylint: disable-msg=C0103
-    def Set (self, interface_name, property_name, # pylint: disable-msg=C0103
+                          in_signature = 'ssv') # pylint: disable=C0103
+    def Set (self, interface_name, property_name, # pylint: disable=C0103
              new_value):
         if interface_name == 'org.mpris.MediaPlayer2':
             raise dbus.exceptions.DBusException (
@@ -218,117 +220,111 @@ class Root (dbus.service.Object): # pylint: disable-msg=R0923,R0904
                 _('The property ‘%s’ is not writeable.'))
         elif interface_name == 'org.mpris.MediaPlayer2.Player':
             if property_name == 'LoopStatus':
-                self.totem.action_remote_set_setting (
+                self.totem.remote_set_setting (
                     Totem.RemoteSetting.REPEAT, (new_value == 'Playlist'))
             elif property_name == 'Rate':
                 # Ignore, since we don't support setting the rate
                 pass
             elif property_name == 'Volume':
-                self.totem.action_volume (new_value)
+                self.totem.set_volume (new_value)
 
             raise dbus.exceptions.DBusException (
                 'org.mpris.MediaPlayer2.ReadOnlyProperty',
                 _('Unknown property ‘%s’ requested of a MediaPlayer 2 object')
-                    % interface_name)
+                % interface_name)
 
         raise dbus.exceptions.DBusException (
             'org.mpris.MediaPlayer2.UnknownInterface',
             _('The MediaPlayer2 object does not implement the ‘%s’ interface')
-                % interface_name)
+            % interface_name)
 
     @dbus.service.signal (dbus_interface = dbus.PROPERTIES_IFACE,
-                          signature = 'sa{sv}as') # pylint: disable-msg=C0103
-    def PropertiesChanged (self, interface_name,  # pylint: disable-msg=C0103
+                          signature = 'sa{sv}as') # pylint: disable=C0103
+    def PropertiesChanged (self, interface_name,  # pylint: disable=C0103
                            changed_properties, invalidated_properties):
         pass
 
     # org.mpris.MediaPlayer2 interface
     @dbus.service.method (dbus_interface = 'org.mpris.MediaPlayer2',
-                          in_signature = '', # pylint: disable-msg=C0103
+                          in_signature = '', # pylint: disable=C0103
                           out_signature = '')
-    def Raise (self): # pylint: disable-msg=C0103
+    def Raise (self): # pylint: disable=C0103
         main_window = self.totem.get_main_window ()
         main_window.present ()
 
     @dbus.service.method (dbus_interface = 'org.mpris.MediaPlayer2',
-                          in_signature = '', # pylint: disable-msg=C0103
+                          in_signature = '', # pylint: disable=C0103
                           out_signature = '')
-    def Quit (self): # pylint: disable-msg=C0103
-        self.totem.action_exit ()
+    def Quit (self): # pylint: disable=C0103
+        self.totem.exit ()
 
     # org.mpris.MediaPlayer2.Player interface
     @dbus.service.method (dbus_interface = 'org.mpris.MediaPlayer2.Player',
-                          in_signature = '', # pylint: disable-msg=C0103
+                          in_signature = '', # pylint: disable=C0103
                           out_signature = '')
-    def Next (self): # pylint: disable-msg=C0103
-        if self.totem.is_playing () or self.totem.is_paused ():
-            return
-
-        self.totem.action_next ()
+    def Next (self): # pylint: disable=C0103
+        self.totem.seek_next ()
 
     @dbus.service.method (dbus_interface = 'org.mpris.MediaPlayer2.Player',
-                          in_signature = '', # pylint: disable-msg=C0103
+                          in_signature = '', # pylint: disable=C0103
                           out_signature = '')
-    def Previous (self): # pylint: disable-msg=C0103
-        if self.totem.is_playing () or self.totem.is_paused ():
-            return
-
-        self.totem.action_previous ()
+    def Previous (self): # pylint: disable=C0103
+        self.totem.seek_previous ()
 
     @dbus.service.method (dbus_interface = 'org.mpris.MediaPlayer2.Player',
-                          in_signature = '', # pylint: disable-msg=C0103
+                          in_signature = '', # pylint: disable=C0103
                           out_signature = '')
-    def Pause (self): # pylint: disable-msg=C0103
-        self.totem.action_pause ()
+    def Pause (self): # pylint: disable=C0103
+        self.totem.pause ()
 
     @dbus.service.method (dbus_interface = 'org.mpris.MediaPlayer2.Player',
-                          in_signature = '', # pylint: disable-msg=C0103
+                          in_signature = '', # pylint: disable=C0103
                           out_signature = '')
-    def PlayPause (self): # pylint: disable-msg=C0103
-        self.totem.action_play_pause ()
+    def PlayPause (self): # pylint: disable=C0103
+        self.totem.play_pause ()
 
     @dbus.service.method (dbus_interface = 'org.mpris.MediaPlayer2.Player',
-                          in_signature = '', # pylint: disable-msg=C0103
+                          in_signature = '', # pylint: disable=C0103
                           out_signature = '')
-    def Stop (self): # pylint: disable-msg=C0103
-        self.totem.action_stop ()
+    def Stop (self): # pylint: disable=C0103
+        self.totem.stop ()
 
     @dbus.service.method (dbus_interface = 'org.mpris.MediaPlayer2.Player',
-                          in_signature = '', # pylint: disable-msg=C0103
+                          in_signature = '', # pylint: disable=C0103
                           out_signature = '')
-    def Play (self): # pylint: disable-msg=C0103
+    def Play (self): # pylint: disable=C0103
         # If playing or no track loaded: do nothing,
         # else: start playing.
-        if self.totem.is_playing () or self.totem.props.current_mrl == None:
+        if self.totem.is_playing () or self.totem.props.current_mrl is None:
             return
 
-        self.totem.action_play ()
+        self.totem.play ()
 
     @dbus.service.method (dbus_interface = 'org.mpris.MediaPlayer2.Player',
-                          in_signature = 'x', # pylint: disable-msg=C0103
+                          in_signature = 'x', # pylint: disable=C0103
                           out_signature = '')
-    def Seek (self, offset): # pylint: disable-msg=C0103
-        self.totem.action_seek_relative (offset / 1000, False)
+    def Seek (self, offset): # pylint: disable=C0103
+        self.totem.seek_relative (offset / 1000, False)
 
     @dbus.service.method (dbus_interface = 'org.mpris.MediaPlayer2.Player',
-                          in_signature = 'ox', # pylint: disable-msg=C0103
+                          in_signature = 'ox', # pylint: disable=C0103
                           out_signature = '')
-    def SetPosition (self, _, position): # pylint: disable-msg=C0103
+    def SetPosition (self, _, position): # pylint: disable=C0103
         position = position / 1000
 
         # Bail if the position is not in the permitted range
         if position < 0 or position > self.totem.props.stream_length:
             return
 
-        self.totem.action_seek_time (position, False)
+        self.totem.seek_time (position, False)
 
     @dbus.service.method (dbus_interface = 'org.mpris.MediaPlayer2.Player',
-                          in_signature = 's', # pylint: disable-msg=C0103
+                          in_signature = 's', # pylint: disable=C0103
                           out_signature = '')
-    def OpenUri (self, uri): # pylint: disable-msg=C0103
+    def OpenUri (self, uri): # pylint: disable=C0103
         self.totem.add_to_playlist_and_play (uri)
 
     @dbus.service.signal (dbus_interface = 'org.mpris.MediaPlayer2.Player',
-                          signature = 'x') # pylint: disable-msg=C0103
-    def Seeked (self, position): # pylint: disable-msg=C0103
+                          signature = 'x') # pylint: disable=C0103
+    def Seeked (self, position): # pylint: disable=C0103
         pass
