@@ -1157,10 +1157,18 @@ content_changed (TotemGrilo   *self,
 	for (i = 0; i < changed_medias->len; i++) {
 		GrlMedia *media = changed_medias->pdata[i];
 		GtkTreeIter *iter;
+		char *str;
+
+		str = grl_media_serialize (media);
+		g_debug ("About to change %s in the store", str);
+		g_free (str);
 
 		if (find_media (model, media, &iter)) {
 			update_media (GTK_TREE_STORE (model), iter, source, media);
 			gtk_tree_iter_free (iter);
+		} else {
+			g_debug ("Could not find '%s' to change in the store",
+				 grl_media_get_id (media));
 		}
 	}
 }
@@ -1188,7 +1196,7 @@ content_removed (TotemGrilo   *self,
 			gtk_tree_store_remove (GTK_TREE_STORE (model), iter);
 			gtk_tree_iter_free (iter);
 		} else {
-			g_debug ("Could not find '%s' in the store",
+			g_debug ("Could not find '%s' to remove in the store",
 				 grl_media_get_id (media));
 		}
 	}
@@ -1209,6 +1217,11 @@ content_added (TotemGrilo   *self,
 
 	for (i = 0; i < changed_medias->len; i++) {
 		GrlMedia *media = changed_medias->pdata[i];
+		char *str;
+
+		str = grl_media_serialize (media);
+		g_debug ("About to add %s to the store", str);
+		g_free (str);
 
 		add_local_metadata (self, source, media);
 		add_media_to_model (GTK_TREE_STORE (model), NULL, source, media);
@@ -1293,7 +1306,9 @@ source_added_cb (GrlRegistry *registry,
 		if (source_is_recent (source)) {
 			browse (self, self->priv->recent_model,
 				NULL, source, NULL, -1);
-			monitor = TRUE;
+			/* https://gitlab.gnome.org/GNOME/grilo-plugins/merge_requests/29 */
+			if (g_str_equal (id, "grl-tracker-source") == FALSE)
+				monitor = TRUE;
 		} else if (!source_is_browse_blacklisted (source)) {
 			const GdkPixbuf *icon;
 
@@ -1411,6 +1426,7 @@ load_grilo_plugins (TotemGrilo *self)
 	GError *error = NULL;
 	GSettings *settings;
 	char **configs;
+	GrlConfig *config;
 	guint i;
 	const char *required_plugins[] = {
 		"grl-lua-factory",
@@ -1427,7 +1443,6 @@ load_grilo_plugins (TotemGrilo *self)
 	g_object_unref (settings);
 
 	for (i = 0; configs[i] != NULL; i++) {
-		GrlConfig *config;
 
 		config = grl_config_new ("grl-filesystem", NULL);
 		grl_config_set_string (config, "base-uri", configs[i]);
@@ -1436,6 +1451,12 @@ load_grilo_plugins (TotemGrilo *self)
 	}
 	g_strfreev (configs);
 
+	/* GStreamer does not support VCDs:
+	 * https://gitlab.freedesktop.org/gstreamer/gst-plugins-bad/merge_requests/203 */
+	config = grl_config_new ("grl-optical-media", NULL);
+	grl_config_set_string (config, "ignored-scheme", "vcd");
+	grl_registry_add_config (registry, config, NULL);
+
 	g_signal_connect (registry, "source-added",
 	                  G_CALLBACK (source_added_cb), self);
 	g_signal_connect (registry, "source-removed",
@@ -2460,7 +2481,7 @@ static void
 setup_config (TotemGrilo *self)
 {
 	GrlRegistry *registry = grl_registry_get_default ();
-	grl_registry_add_config_from_resource (registry, "/org/totem/grilo/totem-grilo.conf", NULL);
+	grl_registry_add_config_from_resource (registry, "/org/gnome/totem/grilo/totem-grilo.conf", NULL);
 }
 
 GtkWidget *
@@ -2771,7 +2792,7 @@ totem_grilo_class_init (TotemGriloClass *klass)
 							   TOTEM_GRILO_PAGE_RECENT, TOTEM_GRILO_PAGE_CHANNELS, TOTEM_GRILO_PAGE_RECENT,
 							   G_PARAM_READWRITE));
 
-	gtk_widget_class_set_template_from_resource (widget_class, "/org/totem/grilo/grilo.ui");
+	gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/totem/grilo/grilo.ui");
 	gtk_widget_class_bind_template_child_private (widget_class, TotemGrilo, selectmenu);
 	gtk_widget_class_bind_template_child_private (widget_class, TotemGrilo, search_bar);
 	gtk_widget_class_bind_template_child_private (widget_class, TotemGrilo, search_entry);
