@@ -375,6 +375,122 @@ escape_label_for_menu (const char *name)
 }
 
 /* Subtitle and language menus */
+static char *
+bvw_lang_info_to_id (BvwLangInfo *info)
+{
+	return g_strdup_printf ("%s-%s", info->language, info->codec);
+}
+
+static int
+hash_table_num_instances (GHashTable *ht,
+			  const char *key)
+{
+	gpointer value;
+
+	value = g_hash_table_lookup (ht, key);
+	if (!value)
+		return 0;
+	return GPOINTER_TO_INT (value);
+}
+
+static const char *
+get_language_name_no_und (const char   *lang,
+			  BvwTrackType  track_type)
+{
+	const char *ret;
+
+	ret =  gst_tag_get_language_name (lang);
+	if (ret != NULL)
+		return ret;
+
+	switch (track_type) {
+	case BVW_TRACK_TYPE_AUDIO:
+		return _("Audio Track");
+		break;
+	case BVW_TRACK_TYPE_SUBTITLE:
+		return _("Subtitle");
+		break;
+	case BVW_TRACK_TYPE_VIDEO:
+		g_assert_not_reached ();
+	}
+
+	return NULL;
+}
+
+GList *
+bvw_lang_info_to_menu_labels (GList        *langs,
+			      BvwTrackType  track_type)
+{
+	GList *l, *ret;
+	GHashTable *lang_table, *lang_codec_table, *printed_table;
+
+	lang_table = g_hash_table_new (g_str_hash, g_str_equal);
+	lang_codec_table = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+
+	/* Populate the hash tables */
+	for (l = langs; l != NULL; l = l->next) {
+		BvwLangInfo *info = l->data;
+		int num;
+		char *id;
+
+		num = hash_table_num_instances (lang_table, info->language);
+		num++;
+		g_hash_table_insert (lang_table,
+				     (gpointer) info->language,
+				     GINT_TO_POINTER (num));
+
+		id = bvw_lang_info_to_id (info);
+		num = hash_table_num_instances (lang_codec_table, id);
+		num++;
+		g_hash_table_insert (lang_codec_table,
+				     id,
+				     GINT_TO_POINTER (num));
+	}
+
+	ret = NULL;
+	printed_table = g_hash_table_new (g_str_hash, g_str_equal);
+	for (l = langs; l != NULL; l = l->next) {
+		BvwLangInfo *info = l->data;
+		int num;
+		char *str;
+
+		num = hash_table_num_instances (lang_table, info->language);
+		g_assert (num >= 1);
+		if (num > 1) {
+			char *id;
+
+			id = bvw_lang_info_to_id (info);
+			num = hash_table_num_instances (lang_codec_table, id);
+			if (num > 1) {
+				num = hash_table_num_instances (printed_table, info->language);
+				num++;
+				g_hash_table_insert (printed_table,
+						     (gpointer) info->language,
+						     GINT_TO_POINTER (num));
+
+				str = g_strdup_printf ("%s #%d",
+						       get_language_name_no_und (info->language, track_type),
+						       num);
+			} else {
+				str = g_strdup_printf ("%s — %s",
+						       get_language_name_no_und (info->language, track_type),
+						       info->codec);
+			}
+			g_free (id);
+		} else {
+			str = g_strdup (get_language_name_no_und (info->language, track_type));
+		}
+
+		ret = g_list_prepend (ret, str);
+	}
+
+	g_hash_table_destroy (printed_table);
+	g_hash_table_destroy (lang_codec_table);
+	g_hash_table_destroy (lang_table);
+
+	return g_list_reverse (ret);
+}
+
 static void
 add_lang_item (GMenu      *menu,
 	       const char *label,
@@ -391,71 +507,39 @@ add_lang_item (GMenu      *menu,
 static void
 add_lang_action (GMenu *menu,
 		 const char *action,
-		 const char *lang,
-		 int lang_id,
-		 int lang_index)
+		 const char *label,
+		 int lang_id)
 {
-	const char *full_lang;
-	char *label;
+	char *escaped_label;
 
-	full_lang = gst_tag_get_language_name (lang);
-
-	if (lang_index > 1) {
-		char *num_lang;
-
-		num_lang = g_strdup_printf ("%s #%u",
-					    full_lang ? full_lang : lang,
-					    lang_index);
-		label = escape_label_for_menu (num_lang);
-		g_free (num_lang);
-	} else {
-		label = escape_label_for_menu (full_lang ? full_lang : lang);
-	}
-
-	add_lang_item (menu, label, action, lang_id);
-	g_free (label);
+	escaped_label = escape_label_for_menu (label);
+	add_lang_item (menu, escaped_label, action, lang_id);
+	g_free (escaped_label);
 }
 
 static void
-create_lang_actions (GMenu *menu,
-		     const char *action,
-		     GList *list,
-		     gboolean is_lang)
+create_lang_actions (GMenu        *menu,
+		     const char   *action,
+		     GList        *list,
+		     BvwTrackType  track_type)
 {
-	unsigned int i;
-	GList *l;
-	GHashTable *lookup;
+	GList *ui_list, *l;
+	guint i;
 
-	if (is_lang == FALSE) {
-		/* Translators: an entry in the "Languages" menu, used to choose the audio language of a DVD */
-		add_lang_action (menu, action, _("None"), -2, 0);
+	if (track_type == BVW_TRACK_TYPE_SUBTITLE) {
+		/* Translators: an entry in the "Subtitles" menu, used to choose the subtitle language of a DVD */
+		add_lang_action (menu, action, _("None"), -2);
 	}
 
 	/* Translators: an entry in the "Languages" menu, used to choose the audio language of a DVD */
-	add_lang_action (menu, action, C_("Language", "Auto"), -1, 0);
+	add_lang_action (menu, action, C_("Language", "Auto"), -1);
 
-	i = 0;
-	lookup = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
+	ui_list = bvw_lang_info_to_menu_labels (list, track_type);
 
-	for (l = list; l != NULL; l = l->next) {
-		guint num;
-		unsigned int *hash_value;
+	for (l = ui_list, i = 0; l != NULL; l = l->next, i++)
+		add_lang_action (menu, action, l->data, i);
 
-		hash_value = g_hash_table_lookup (lookup, l->data);
-		if (hash_value == NULL)
-			num = 0;
-		else
-			num = GPOINTER_TO_INT (hash_value);
-		num++;
-
-		g_hash_table_insert (lookup, l->data, GINT_TO_POINTER (num));
-
-		add_lang_action (menu, action, l->data, i, num);
-
-		i++;
-	}
-
-	g_hash_table_destroy (lookup);
+	g_list_free_full (ui_list, (GDestroyNotify) g_free);
 }
 
 static gboolean
@@ -475,9 +559,14 @@ totem_sublang_equal_lists (GList *orig, GList *new)
 	retval = TRUE;
 	o = orig;
 	n = new;
-	while (o != NULL && n != NULL && retval != FALSE)
-	{
-		if (g_str_equal (o->data, n->data) == FALSE)
+	while (o != NULL && n != NULL && retval != FALSE) {
+		BvwLangInfo *info_o, *info_n;
+
+		info_o = o->data;
+		info_n = n->data;
+		if (g_strcmp0 (info_o->language, info_n->language) != 0)
+			retval = FALSE;
+		if (g_strcmp0 (info_o->codec, info_n->codec) != 0)
 			retval = FALSE;
                 o = g_list_next (o);
                 n = g_list_next (n);
@@ -498,7 +587,7 @@ totem_languages_update (Totem *totem, GList *list)
 	if (list != NULL) {
 		GMenu *menu;
 		menu = totem_object_get_menu_section (totem, "languages-placeholder");
-		create_lang_actions (menu, "app.set-language", list, TRUE);
+		create_lang_actions (menu, "app.set-language", list, BVW_TRACK_TYPE_AUDIO);
 	}
 
 	action = g_action_map_lookup_action (G_ACTION_MAP (totem), "set-language");
@@ -507,7 +596,7 @@ totem_languages_update (Totem *totem, GList *list)
 	g_action_change_state (action, g_variant_new_int32 (current));
 	totem->updating_menu = FALSE;
 
-	g_list_free_full (totem->languages_list, g_free);
+	g_list_free_full (totem->languages_list, (GDestroyNotify) bacon_video_widget_lang_info_free);
 	totem->languages_list = list;
 }
 
@@ -523,7 +612,7 @@ totem_subtitles_update (Totem *totem, GList *list)
 	if (list != NULL) {
 		GMenu *menu;
 		menu = totem_object_get_menu_section (totem, "subtitles-placeholder");
-		create_lang_actions (menu, "app.set-subtitle", list, FALSE);
+		create_lang_actions (menu, "app.set-subtitle", list, BVW_TRACK_TYPE_SUBTITLE);
 	}
 
 	action = g_action_map_lookup_action (G_ACTION_MAP (totem), "set-subtitle");
@@ -532,7 +621,7 @@ totem_subtitles_update (Totem *totem, GList *list)
 	g_action_change_state (action, g_variant_new_int32 (current));
 	totem->updating_menu = FALSE;
 
-	g_list_free_full (totem->subtitles_list, g_free);
+	g_list_free_full (totem->subtitles_list, (GDestroyNotify) bacon_video_widget_lang_info_free);
 	totem->subtitles_list = list;
 }
 
@@ -543,14 +632,14 @@ totem_sublang_update (Totem *totem)
 
 	list = bacon_video_widget_get_languages (totem->bvw);
 	if (totem_sublang_equal_lists (totem->languages_list, list) == TRUE) {
-		g_list_free_full (list, g_free);
+		g_list_free_full (list, (GDestroyNotify) bacon_video_widget_lang_info_free);
 	} else {
 		totem_languages_update (totem, list);
 	}
 
 	list = bacon_video_widget_get_subtitles (totem->bvw);
 	if (totem_sublang_equal_lists (totem->subtitles_list, list) == TRUE) {
-		g_list_free_full (list, g_free);
+		g_list_free_full (list, (GDestroyNotify) bacon_video_widget_lang_info_free);
 	} else {
 		totem_subtitles_update (totem, list);
 	}
@@ -559,6 +648,6 @@ totem_sublang_update (Totem *totem)
 void
 totem_sublang_exit (Totem *totem)
 {
-	g_list_free_full (totem->subtitles_list, g_free);
-	g_list_free_full (totem->languages_list, g_free);
+	g_list_free_full (totem->subtitles_list, (GDestroyNotify) bacon_video_widget_lang_info_free);
+	g_list_free_full (totem->languages_list, (GDestroyNotify) bacon_video_widget_lang_info_free);
 }
